#!/usr/bin/env ruby

# This example demonstrates the use of 2D text.

require 'vtk'

# Create a sphere source, mapper, && actor
sphere = Vtk::SphereSource.new

sphereMapper = Vtk::PolyDataMapper.new
sphereMapper.SetInputConnection(sphere.GetOutputPort)
Vtk::PolyDataMapper.GlobalImmediateModeRenderingOn
sphereActor = Vtk::LODActor.new
sphereActor.SetMapper(sphereMapper)

# Create a scaled text actor. 
# Set the text, font, justification, && properties (bold, italics,
# etc.).
textActor = Vtk::TextActor.new
textActor.ScaledTextOn
textActor.SetDisplayPosition(90, 50)
textActor.SetInput("This is a sphere")

# Set coordinates to match the old vtkScaledTextActor default value
textActor.GetPosition2Coordinate.SetCoordinateSystemToNormalizedViewport
textActor.GetPosition2Coordinate.SetValue(0.6, 0.1)

tprop = textActor.GetTextProperty
tprop.SetFontSize(18)
tprop.SetFontFamilyToArial
tprop.SetJustificationToCentered
tprop.BoldOn
tprop.ItalicOn
tprop.ShadowOn
tprop.SetColor(0, 0, 1)

# Create the Renderer, RenderWindow, RenderWindowInteractor
ren = Vtk::Renderer.new
renWin = Vtk::RenderWindow.new
renWin.AddRenderer(ren)
iren = Vtk::RenderWindowInteractor.new
iren.SetRenderWindow(renWin)

# Add the actors to the renderer; set the background && size; zoom
# in; && render.
ren.AddActor2D(textActor)
ren.AddActor(sphereActor)

ren.SetBackground(1, 1, 1)
renWin.SetSize(250, 125)
ren.ResetCamera
ren.GetActiveCamera.Zoom(1.5)

iren.Initialize
renWin.Render
iren.Start
