#!/usr/bin/env ruby

# This example demonstrates the use of 2D text the old way by using a
# vtkTextMapper && a vtkScaledTextActor.

require 'vtk'

# Create a sphere source, mapper, && actor
sphere = Vtk::SphereSource.new

sphereMapper = Vtk::PolyDataMapper.new
sphereMapper.SetInputConnection(sphere.GetOutputPort)
Vtk::PolyDataMapper.GlobalImmediateModeRenderingOn
sphereActor = Vtk::LODActor.new
sphereActor.SetMapper(sphereMapper)

# Create a text mapper.
textMapper = Vtk::TextMapper.new
textMapper.SetInput("This is a sphere")

# Set the text, font, justification, && text properties (bold,
# italics, etc.).
tprop = textMapper.GetTextProperty
tprop.SetFontSize(18)
tprop.SetFontFamilyToArial
tprop.SetJustificationToCentered
tprop.BoldOn
tprop.ItalicOn
tprop.ShadowOn
tprop.SetColor(0, 0, 1)

# Create a scaled text actor. Set the position of the text.
textActor = Vtk::ScaledTextActor.new
textActor.SetMapper(textMapper)
textActor.SetDisplayPosition(90, 50)

# Create the Renderer, RenderWindow, RenderWindowInteractor
ren = Vtk::Renderer.new
renWin = Vtk::RenderWindow.new
renWin.AddRenderer(ren)
iren = Vtk::RenderWindowInteractor.new
iren.SetRenderWindow(renWin)

# Add the actors to the renderer; set the background && size; zoom
# in; && render.
ren.AddActor2D(textActor)
ren.AddActor(sphereActor)

ren.SetBackground(1, 1, 1)
renWin.SetSize(250, 125)
ren.ResetCamera
ren.GetActiveCamera.Zoom(1.5)

iren.Initialize
renWin.Render
iren.Start
