#!/usr/bin/env ruby

# This example demonstrates the use of vtkVectorText && vtkFollower.
# vtkVectorText is used to create 3D annotation.  vtkFollower is used to
# position the 3D text && to ensure that the text always faces the
# renderer's active camera (i.e., the text is always readable).

require 'vtk'

# Create the axes && the associated mapper && actor.
axes = Vtk::Axes.new
axes.SetOrigin(0, 0, 0)
axesMapper = Vtk::PolyDataMapper.new
axesMapper.SetInputConnection(axes.GetOutputPort)
axesActor = Vtk::Actor.new
axesActor.SetMapper(axesMapper)

# Create the 3D text && the associated mapper && follower (a type of
# actor).  Position the text so it is displayed over the origin of the
# axes.
atext = Vtk::VectorText.new
atext.SetText("Origin")
textMapper = Vtk::PolyDataMapper.new
textMapper.SetInputConnection(atext.GetOutputPort)
textActor = Vtk::Follower.new
textActor.SetMapper(textMapper)
textActor.SetScale(0.2, 0.2, 0.2)
textActor.AddPosition(0, -0.1, 0)

# Create the Renderer, RenderWindow, && RenderWindowInteractor.
ren = Vtk::Renderer.new
renWin = Vtk::RenderWindow.new
renWin.AddRenderer(ren)
iren = Vtk::RenderWindowInteractor.new
iren.SetRenderWindow(renWin)

# Add the actors to the renderer.
ren.AddActor(axesActor)
ren.AddActor(textActor)

# Zoom in closer.
ren.ResetCamera
ren.GetActiveCamera.Zoom(1.6)

# Reset the clipping range of the camera; set the camera of the
# follower; render.
ren.ResetCameraClippingRange
textActor.SetCamera(ren.GetActiveCamera)

iren.Initialize
renWin.Render
iren.Start
