#!/usr/bin/env ruby

# This example shows how to manually construct unstructured grids
# using Ruby.  Unstructured grids require explicit point && cell
# representations, so every point && cell must be created, && then
# added to the vtkUnstructuredGrid instance.

require 'vtk'

# Create several unstructured grids each containing a cell of a
# different type.
voxelPoints = Vtk::Points.new
voxelPoints.SetNumberOfPoints(8)
voxelPoints.InsertPoint(0, 0, 0, 0)
voxelPoints.InsertPoint(1, 1, 0, 0)
voxelPoints.InsertPoint(2, 0, 1, 0)
voxelPoints.InsertPoint(3, 1, 1, 0)
voxelPoints.InsertPoint(4, 0, 0, 1)
voxelPoints.InsertPoint(5, 1, 0, 1)
voxelPoints.InsertPoint(6, 0, 1, 1)
voxelPoints.InsertPoint(7, 1, 1, 1)
aVoxel = Vtk::Voxel.new
aVoxel.GetPointIds.SetId(0, 0)
aVoxel.GetPointIds.SetId(1, 1)
aVoxel.GetPointIds.SetId(2, 2)
aVoxel.GetPointIds.SetId(3, 3)
aVoxel.GetPointIds.SetId(4, 4)
aVoxel.GetPointIds.SetId(5, 5)
aVoxel.GetPointIds.SetId(6, 6)
aVoxel.GetPointIds.SetId(7, 7)
aVoxelGrid = Vtk::UnstructuredGrid.new
aVoxelGrid.Allocate(1, 1)
aVoxelGrid.InsertNextCell(aVoxel.GetCellType, aVoxel.GetPointIds)
aVoxelGrid.SetPoints(voxelPoints)
aVoxelMapper = Vtk::DataSetMapper.new
aVoxelMapper.SetInput(aVoxelGrid)
aVoxelActor = Vtk::Actor.new
aVoxelActor.SetMapper(aVoxelMapper)
aVoxelActor.GetProperty.SetDiffuseColor(1, 0, 0)

hexahedronPoints = Vtk::Points.new
hexahedronPoints.SetNumberOfPoints(8)
hexahedronPoints.InsertPoint(0, 0, 0, 0)
hexahedronPoints.InsertPoint(1, 1, 0, 0)
hexahedronPoints.InsertPoint(2, 1, 1, 0)
hexahedronPoints.InsertPoint(3, 0, 1, 0)
hexahedronPoints.InsertPoint(4, 0, 0, 1)
hexahedronPoints.InsertPoint(5, 1, 0, 1)
hexahedronPoints.InsertPoint(6, 1, 1, 1)
hexahedronPoints.InsertPoint(7, 0, 1, 1)
aHexahedron = Vtk::Hexahedron.new
aHexahedron.GetPointIds.SetId(0, 0)
aHexahedron.GetPointIds.SetId(1, 1)
aHexahedron.GetPointIds.SetId(2, 2)
aHexahedron.GetPointIds.SetId(3, 3)
aHexahedron.GetPointIds.SetId(4, 4)
aHexahedron.GetPointIds.SetId(5, 5)
aHexahedron.GetPointIds.SetId(6, 6)
aHexahedron.GetPointIds.SetId(7, 7)
aHexahedronGrid = Vtk::UnstructuredGrid.new
aHexahedronGrid.Allocate(1, 1)
aHexahedronGrid.InsertNextCell(aHexahedron.GetCellType,
                               aHexahedron.GetPointIds)
aHexahedronGrid.SetPoints(hexahedronPoints)
aHexahedronMapper = Vtk::DataSetMapper.new
aHexahedronMapper.SetInput(aHexahedronGrid)
aHexahedronActor = Vtk::Actor.new
aHexahedronActor.SetMapper(aHexahedronMapper)
aHexahedronActor.AddPosition(2, 0, 0)
aHexahedronActor.GetProperty.SetDiffuseColor(1, 1, 0)

tetraPoints = Vtk::Points.new
tetraPoints.SetNumberOfPoints(4)
tetraPoints.InsertPoint(0, 0, 0, 0)
tetraPoints.InsertPoint(1, 1, 0, 0)
tetraPoints.InsertPoint(2, 0.5, 1, 0)
tetraPoints.InsertPoint(3, 0.5, 0.5, 1)
aTetra = Vtk::Tetra.new
aTetra.GetPointIds.SetId(0, 0)
aTetra.GetPointIds.SetId(1, 1)
aTetra.GetPointIds.SetId(2, 2)
aTetra.GetPointIds.SetId(3, 3)
aTetraGrid = Vtk::UnstructuredGrid.new
aTetraGrid.Allocate(1, 1)
aTetraGrid.InsertNextCell(aTetra.GetCellType, aTetra.GetPointIds)
aTetraGrid.SetPoints(tetraPoints)
aTetraMapper = Vtk::DataSetMapper.new
aTetraMapper.SetInput(aTetraGrid)
aTetraActor = Vtk::Actor.new
aTetraActor.SetMapper(aTetraMapper)
aTetraActor.AddPosition(4, 0, 0)
aTetraActor.GetProperty.SetDiffuseColor(0, 1, 0)

wedgePoints = Vtk::Points.new
wedgePoints.SetNumberOfPoints(6)
wedgePoints.InsertPoint(0, 0, 1, 0)
wedgePoints.InsertPoint(1, 0, 0, 0)
wedgePoints.InsertPoint(2, 0, 0.5, 0.5)
wedgePoints.InsertPoint(3, 1, 1, 0)
wedgePoints.InsertPoint(4, 1, 0, 0)
wedgePoints.InsertPoint(5, 1, 0.5, 0.5)
aWedge = Vtk::Wedge.new
aWedge.GetPointIds.SetId(0, 0)
aWedge.GetPointIds.SetId(1, 1)
aWedge.GetPointIds.SetId(2, 2)
aWedge.GetPointIds.SetId(3, 3)
aWedge.GetPointIds.SetId(4, 4)
aWedge.GetPointIds.SetId(5, 5)
aWedgeGrid = Vtk::UnstructuredGrid.new
aWedgeGrid.Allocate(1, 1)
aWedgeGrid.InsertNextCell(aWedge.GetCellType, aWedge.GetPointIds)
aWedgeGrid.SetPoints(wedgePoints)
aWedgeMapper = Vtk::DataSetMapper.new
aWedgeMapper.SetInput(aWedgeGrid)
aWedgeActor = Vtk::Actor.new
aWedgeActor.SetMapper(aWedgeMapper)
aWedgeActor.AddPosition(6, 0, 0)
aWedgeActor.GetProperty.SetDiffuseColor(0, 1, 1)

pyramidPoints = Vtk::Points.new
pyramidPoints.SetNumberOfPoints(5)
pyramidPoints.InsertPoint(0, 0, 0, 0)
pyramidPoints.InsertPoint(1, 1, 0, 0)
pyramidPoints.InsertPoint(2, 1, 1, 0)
pyramidPoints.InsertPoint(3, 0, 1, 0)
pyramidPoints.InsertPoint(4, 0.5, 0.5, 1)
aPyramid = Vtk::Pyramid.new
aPyramid.GetPointIds.SetId(0, 0)
aPyramid.GetPointIds.SetId(1, 1)
aPyramid.GetPointIds.SetId(2, 2)
aPyramid.GetPointIds.SetId(3, 3)
aPyramid.GetPointIds.SetId(4, 4)
aPyramidGrid = Vtk::UnstructuredGrid.new
aPyramidGrid.Allocate(1, 1)
aPyramidGrid.InsertNextCell(aPyramid.GetCellType, aPyramid.GetPointIds)
aPyramidGrid.SetPoints(pyramidPoints)
aPyramidMapper = Vtk::DataSetMapper.new
aPyramidMapper.SetInput(aPyramidGrid)
aPyramidActor = Vtk::Actor.new
aPyramidActor.SetMapper(aPyramidMapper)
aPyramidActor.AddPosition(8, 0, 0)
aPyramidActor.GetProperty.SetDiffuseColor(1, 0, 1)

pixelPoints = Vtk::Points.new
pixelPoints.SetNumberOfPoints(4)
pixelPoints.InsertPoint(0, 0, 0, 0)
pixelPoints.InsertPoint(1, 1, 0, 0)
pixelPoints.InsertPoint(2, 0, 1, 0)
pixelPoints.InsertPoint(3, 1, 1, 0)
aPixel = Vtk::Pixel.new
aPixel.GetPointIds.SetId(0, 0)
aPixel.GetPointIds.SetId(1, 1)
aPixel.GetPointIds.SetId(2, 2)
aPixel.GetPointIds.SetId(3, 3)
aPixelGrid = Vtk::UnstructuredGrid.new
aPixelGrid.Allocate(1, 1)
aPixelGrid.InsertNextCell(aPixel.GetCellType, aPixel.GetPointIds)
aPixelGrid.SetPoints(pixelPoints)
aPixelMapper = Vtk::DataSetMapper.new
aPixelMapper.SetInput(aPixelGrid)
aPixelActor = Vtk::Actor.new
aPixelActor.SetMapper(aPixelMapper)
aPixelActor.AddPosition(0, 0, 2)
aPixelActor.GetProperty.SetDiffuseColor(0, 1, 1)

quadPoints = Vtk::Points.new
quadPoints.SetNumberOfPoints(4)
quadPoints.InsertPoint(0, 0, 0, 0)
quadPoints.InsertPoint(1, 1, 0, 0)
quadPoints.InsertPoint(2, 1, 1, 0)
quadPoints.InsertPoint(3, 0, 1, 0)
aQuad = Vtk::Quad.new
aQuad.GetPointIds.SetId(0, 0)
aQuad.GetPointIds.SetId(1, 1)
aQuad.GetPointIds.SetId(2, 2)
aQuad.GetPointIds.SetId(3, 3)
aQuadGrid = Vtk::UnstructuredGrid.new
aQuadGrid.Allocate(1, 1)
aQuadGrid.InsertNextCell(aQuad.GetCellType, aQuad.GetPointIds)
aQuadGrid.SetPoints(quadPoints)
aQuadMapper = Vtk::DataSetMapper.new
aQuadMapper.SetInput(aQuadGrid)
aQuadActor = Vtk::Actor.new
aQuadActor.SetMapper(aQuadMapper)
aQuadActor.AddPosition(2, 0, 2)
aQuadActor.GetProperty.SetDiffuseColor(1, 0, 1)

trianglePoints = Vtk::Points.new
trianglePoints.SetNumberOfPoints(3)
trianglePoints.InsertPoint(0, 0, 0, 0)
trianglePoints.InsertPoint(1, 1, 0, 0)
trianglePoints.InsertPoint(2, 0.5, 0.5, 0)
triangleTCoords = Vtk::FloatArray.new
triangleTCoords.SetNumberOfComponents(3)
triangleTCoords.SetNumberOfTuples(3)
triangleTCoords.InsertTuple3(0, 1, 1, 1)
triangleTCoords.InsertTuple3(1, 2, 2, 2)
triangleTCoords.InsertTuple3(2, 3, 3, 3)
aTriangle = Vtk::Triangle.new
aTriangle.GetPointIds.SetId(0, 0)
aTriangle.GetPointIds.SetId(1, 1)
aTriangle.GetPointIds.SetId(2, 2)
aTriangleGrid = Vtk::UnstructuredGrid.new
aTriangleGrid.Allocate(1, 1)
aTriangleGrid.InsertNextCell(aTriangle.GetCellType,
                             aTriangle.GetPointIds)
aTriangleGrid.SetPoints(trianglePoints)
aTriangleGrid.GetPointData.SetTCoords(triangleTCoords)
aTriangleMapper = Vtk::DataSetMapper.new
aTriangleMapper.SetInput(aTriangleGrid)
aTriangleActor = Vtk::Actor.new
aTriangleActor.SetMapper(aTriangleMapper)
aTriangleActor.AddPosition(4, 0, 2)
aTriangleActor.GetProperty.SetDiffuseColor(0.3, 1, 0.5)

polygonPoints = Vtk::Points.new
polygonPoints.SetNumberOfPoints(4)
polygonPoints.InsertPoint(0, 0, 0, 0)
polygonPoints.InsertPoint(1, 1, 0, 0)
polygonPoints.InsertPoint(2, 1, 1, 0)
polygonPoints.InsertPoint(3, 0, 1, 0)
aPolygon = Vtk::Polygon.new
aPolygon.GetPointIds.SetNumberOfIds(4)
aPolygon.GetPointIds.SetId(0, 0)
aPolygon.GetPointIds.SetId(1, 1)
aPolygon.GetPointIds.SetId(2, 2)
aPolygon.GetPointIds.SetId(3, 3)
aPolygonGrid = Vtk::UnstructuredGrid.new
aPolygonGrid.Allocate(1, 1)
aPolygonGrid.InsertNextCell(aPolygon.GetCellType, aPolygon.GetPointIds)
aPolygonGrid.SetPoints(polygonPoints)
aPolygonMapper = Vtk::DataSetMapper.new
aPolygonMapper.SetInput(aPolygonGrid)
aPolygonActor = Vtk::Actor.new
aPolygonActor.SetMapper(aPolygonMapper)
aPolygonActor.AddPosition(6, 0, 2)
aPolygonActor.GetProperty.SetDiffuseColor(1, 0.4, 0.5)

triangleStripPoints = Vtk::Points.new
triangleStripPoints.SetNumberOfPoints(5)
triangleStripPoints.InsertPoint(0, 0, 1, 0)
triangleStripPoints.InsertPoint(1, 0, 0, 0)
triangleStripPoints.InsertPoint(2, 1, 1, 0)
triangleStripPoints.InsertPoint(3, 1, 0, 0)
triangleStripPoints.InsertPoint(4, 2, 1, 0)
triangleStripTCoords = Vtk::FloatArray.new
triangleStripTCoords.SetNumberOfComponents(3)
triangleStripTCoords.SetNumberOfTuples(3)
triangleStripTCoords.InsertTuple3(0, 1, 1, 1)
triangleStripTCoords.InsertTuple3(1, 2, 2, 2)
triangleStripTCoords.InsertTuple3(2, 3, 3, 3)
triangleStripTCoords.InsertTuple3(3, 4, 4, 4)
triangleStripTCoords.InsertTuple3(4, 5, 5, 5)
aTriangleStrip = Vtk::TriangleStrip.new
aTriangleStrip.GetPointIds.SetNumberOfIds(5)
aTriangleStrip.GetPointIds.SetId(0, 0)
aTriangleStrip.GetPointIds.SetId(1, 1)
aTriangleStrip.GetPointIds.SetId(2, 2)
aTriangleStrip.GetPointIds.SetId(3, 3)
aTriangleStrip.GetPointIds.SetId(4, 4)
aTriangleStripGrid = Vtk::UnstructuredGrid.new
aTriangleStripGrid.Allocate(1, 1)
aTriangleStripGrid.InsertNextCell(aTriangleStrip.GetCellType,
                                  aTriangleStrip.GetPointIds)
aTriangleStripGrid.SetPoints(triangleStripPoints)
aTriangleStripGrid.GetPointData.SetTCoords(triangleStripTCoords)
aTriangleStripMapper = Vtk::DataSetMapper.new
aTriangleStripMapper.SetInput(aTriangleStripGrid)
aTriangleStripActor = Vtk::Actor.new
aTriangleStripActor.SetMapper(aTriangleStripMapper)
aTriangleStripActor.AddPosition(8, 0, 2)
aTriangleStripActor.GetProperty.SetDiffuseColor(0.3, 0.7, 1)

linePoints = Vtk::Points.new
linePoints.SetNumberOfPoints(2)
linePoints.InsertPoint(0, 0, 0, 0)
linePoints.InsertPoint(1, 1, 1, 0)
aLine = Vtk::Line.new
aLine.GetPointIds.SetId(0, 0)
aLine.GetPointIds.SetId(1, 1)
aLineGrid = Vtk::UnstructuredGrid.new
aLineGrid.Allocate(1, 1)
aLineGrid.InsertNextCell(aLine.GetCellType, aLine.GetPointIds)
aLineGrid.SetPoints(linePoints)
aLineMapper = Vtk::DataSetMapper.new
aLineMapper.SetInput(aLineGrid)
aLineActor = Vtk::Actor.new
aLineActor.SetMapper(aLineMapper)
aLineActor.AddPosition(0, 0, 4)
aLineActor.GetProperty.SetDiffuseColor(0.2, 1, 1)

polyLinePoints = Vtk::Points.new
polyLinePoints.SetNumberOfPoints(3)
polyLinePoints.InsertPoint(0, 0, 0, 0)
polyLinePoints.InsertPoint(1, 1, 1, 0)
polyLinePoints.InsertPoint(2, 1, 0, 0)
aPolyLine = Vtk::PolyLine.new
aPolyLine.GetPointIds.SetNumberOfIds(3)
aPolyLine.GetPointIds.SetId(0, 0)
aPolyLine.GetPointIds.SetId(1, 1)
aPolyLine.GetPointIds.SetId(2, 2)
aPolyLineGrid = Vtk::UnstructuredGrid.new
aPolyLineGrid.Allocate(1, 1)
aPolyLineGrid.InsertNextCell(aPolyLine.GetCellType,
                             aPolyLine.GetPointIds)
aPolyLineGrid.SetPoints(polyLinePoints)
aPolyLineMapper = Vtk::DataSetMapper.new
aPolyLineMapper.SetInput(aPolyLineGrid)
aPolyLineActor = Vtk::Actor.new
aPolyLineActor.SetMapper(aPolyLineMapper)
aPolyLineActor.AddPosition(2, 0, 4)
aPolyLineActor.GetProperty.SetDiffuseColor(1, 1, 1)

vertexPoints = Vtk::Points.new
vertexPoints.SetNumberOfPoints(1)
vertexPoints.InsertPoint(0, 0, 0, 0)
aVertex = Vtk::Vertex.new
aVertex.GetPointIds.SetId(0, 0)
aVertexGrid = Vtk::UnstructuredGrid.new
aVertexGrid.Allocate(1, 1)
aVertexGrid.InsertNextCell(aVertex.GetCellType, aVertex.GetPointIds)
aVertexGrid.SetPoints(vertexPoints)
aVertexMapper = Vtk::DataSetMapper.new
aVertexMapper.SetInput(aVertexGrid)
aVertexActor = Vtk::Actor.new
aVertexActor.SetMapper(aVertexMapper)
aVertexActor.AddPosition(0, 0, 6)
aVertexActor.GetProperty.SetDiffuseColor(1, 1, 1)

polyVertexPoints = Vtk::Points.new
polyVertexPoints.SetNumberOfPoints(3)
polyVertexPoints.InsertPoint(0, 0, 0, 0)
polyVertexPoints.InsertPoint(1, 1, 0, 0)
polyVertexPoints.InsertPoint(2, 1, 1, 0)
aPolyVertex = Vtk::PolyVertex.new
aPolyVertex.GetPointIds.SetNumberOfIds(3)
aPolyVertex.GetPointIds.SetId(0, 0)
aPolyVertex.GetPointIds.SetId(1, 1)
aPolyVertex.GetPointIds.SetId(2, 2)
aPolyVertexGrid = Vtk::UnstructuredGrid.new
aPolyVertexGrid.Allocate(1, 1)
aPolyVertexGrid.InsertNextCell(aPolyVertex.GetCellType,
                               aPolyVertex.GetPointIds)
aPolyVertexGrid.SetPoints(polyVertexPoints)
aPolyVertexMapper = Vtk::DataSetMapper.new
aPolyVertexMapper.SetInput(aPolyVertexGrid)
aPolyVertexActor = Vtk::Actor.new
aPolyVertexActor.SetMapper(aPolyVertexMapper)
aPolyVertexActor.AddPosition(2, 0, 6)
aPolyVertexActor.GetProperty.SetDiffuseColor(1, 1, 1)

# Create the usual rendering stuff.
ren = Vtk::Renderer.new
renWin = Vtk::RenderWindow.new
renWin.AddRenderer(ren)
renWin.SetSize(300, 150)
iren = Vtk::RenderWindowInteractor.new
iren.SetRenderWindow(renWin)

ren.SetBackground(0.1, 0.2, 0.4)

ren.AddActor(aVoxelActor)
ren.AddActor(aHexahedronActor)
ren.AddActor(aTetraActor)
ren.AddActor(aWedgeActor)
ren.AddActor(aPyramidActor)
ren.AddActor(aPixelActor)
ren.AddActor(aQuadActor)
ren.AddActor(aTriangleActor)
ren.AddActor(aPolygonActor)
ren.AddActor(aTriangleStripActor)
ren.AddActor(aLineActor)
ren.AddActor(aPolyLineActor)
ren.AddActor(aVertexActor)
ren.AddActor(aPolyVertexActor)

ren.ResetCamera
ren.GetActiveCamera.Azimuth(30)
ren.GetActiveCamera.Elevation(20)
ren.GetActiveCamera.Dolly(2.8)
ren.ResetCameraClippingRange

# Render the scene && start interaction.
iren.Initialize
renWin.Render
iren.Start
