#!/usr/bin/env ruby

# This example demonstrates how to use the vtkPlaneWidget to probe a
# dataset && then generate contours on the probed data.

require 'vtk'

# Create a mace out of filters.
sphere = Vtk::SphereSource.new
cone = Vtk::ConeSource.new
glyph = Vtk::Glyph3D.new
glyph.SetInputConnection(sphere.GetOutputPort)
glyph.SetSource(cone.GetOutput)
glyph.SetVectorModeToUseNormal
glyph.SetScaleModeToScaleByVector
glyph.SetScaleFactor(0.25)

# The sphere && spikes are appended into a single polydata.
# This just makes things simpler to manage.
apd = Vtk::AppendPolyData.new
apd.AddInput(glyph.GetOutput)
apd.AddInput(sphere.GetOutput)

maceMapper = Vtk::PolyDataMapper.new
maceMapper.SetInputConnection(apd.GetOutputPort)

maceActor = Vtk::LODActor.new
maceActor.SetMapper(maceMapper)
maceActor.VisibilityOn

# This portion of the code clips the mace with the vtkPlanes
# implicit function. The clipped region is colored green.
plane = Vtk::Plane.new
clipper = Vtk::ClipPolyData.new
clipper.SetInputConnection(apd.GetOutputPort)
clipper.SetClipFunction(plane)
clipper.InsideOutOn

selectMapper = Vtk::PolyDataMapper.new
selectMapper.SetInputConnection(clipper.GetOutputPort)

selectActor = Vtk::LODActor.new
selectActor.SetMapper(selectMapper)
selectActor.GetProperty.SetColor(0, 1, 0)
selectActor.VisibilityOff
selectActor.SetScale(1.01, 1.01, 1.01)

# Create the RenderWindow, Renderer && both Actors
ren = Vtk::Renderer.new
renWin = Vtk::RenderWindow.new
renWin.AddRenderer(ren)
iren = Vtk::RenderWindowInteractor.new
iren.SetRenderWindow(renWin)

# The callback function
myCallback = Proc.new{|obj, event|
    obj.GetPlane(plane)
    selectActor.VisibilityOn
}

# Associate the line widget with the interactor
planeWidget = Vtk::ImplicitPlaneWidget.new
planeWidget.SetInteractor(iren)
planeWidget.SetPlaceFactor(1.25)
planeWidget.SetInput(glyph.GetOutput)
planeWidget.PlaceWidget
planeWidget.AddObserver("InteractionEvent", myCallback)

ren.AddActor(maceActor)
ren.AddActor(selectActor)

# Add the actors to the renderer, set the background && size
ren.SetBackground(1, 1, 1)
renWin.SetSize(300, 300)
ren.SetBackground(0.1, 0.2, 0.4)

# Start interaction.
iren.Initialize
renWin.Render
iren.Start
