require 'vtk'
require 'vtk/util'
require 'vtk/tk'


class SampleViewer
  def initialize
    root = TkRoot.new.title( 'Ruby Version of vtkImageDataToTkPhoto' )

    # Image pipeline
    reader = Vtk::Volume16Reader.new
    reader.SetDataDimensions( 64, 64 )
    reader.SetDataByteOrderToLittleEndian
    reader.SetFilePrefix( vtkGetDataRoot + '/Data/headsq/quarter'  )
    reader.SetImageRange( 1, 93 )
    reader.SetDataSpacing( 3.2, 3.2, 1.5 )
    reader.Update 

    @cast = cast = Vtk::ImageCast.new
    cast.SetInputConnection( reader.GetOutputPort )
    cast.SetOutputScalarType( reader.GetOutput.GetScalarType )
    cast.ClampOverflowOn

    # Make the image a little bigger
    @resample = resample = Vtk::ImageResample.new
    resample.SetInputConnection( cast.GetOutputPort )
    resample.SetAxisMagnificationFactor( 0, 2 )
    resample.SetAxisMagnificationFactor( 1, 2 )
    resample.SetAxisMagnificationFactor( 2, 1 )

    l,h = reader.GetOutput.GetScalarRange

    # Create the three orthogonal views

    tphoto = @tphoto = Vtk::TkPhotoImage.new
    cphoto = @cphoto = Vtk::TkPhotoImage.new
    sphoto = @sphoto = Vtk::TkPhotoImage.new
    reader.Update
    d = reader.GetOutput.GetDimensions
    @Position = [ (d[0]/2.0).to_i, (d[0]/2.0).to_i, (d[0]/2.0).to_i ]

    # Create a popup menu
    castToUnsignedChar = Proc.new{
      @cast.SetOutputScalarTypeToUnsignedChar
      self.SetImages
    }
    castToUnsignedShort = Proc.new{
      @cast.SetOutputScalarTypeToUnsignedShort
      self.SetImages
    }
    castToUnsignedInt = Proc.new{
      @cast.SetOutputScalarTypeToUnsignedInt
      self.SetImages
    }
    castToFloat = Proc.new{
      @cast.SetOutputScalarTypeToFloat
      self.SetImages
    }

    v = TkVariable.new
    @popup = popup = TkMenu.new( root, 'tearoff'=>0 )
    popup.add_radiobutton( 'label'=>'unsigned char', 'command'=>castToUnsignedChar, 'variable'=>v, 'value'=>-1 )
    popup.add_radiobutton( 'label'=>'unsigned short', 'command'=>castToUnsignedShort, 'variable'=>v, 'value'=>0 )
    popup.add_radiobutton( 'label'=>'unsigned int', 'command'=>castToUnsignedInt, 'variable'=>v, 'value'=>1 )
    popup.add_radiobutton( 'label'=>'float', 'command'=>castToFloat, 'variable'=>v, 'value'=>2 )

    v.value = 0

    setWindowLevel = Proc.new{|event|
      self.SetImages
    }

    w = @TransverseLabelWidget = TkLabel.new( root, 'image'=>tphoto )
    w.grid( 'row'=>0, 'column'=>0 )
    w.bind( "Button1-Motion" ){|e,i,o.s| i||i=tphoto; o||o='transverse'; s||s=self; s.Motion( e, i, o ) }
    w.bind( "Button-3" ){|e| self.DoPopup(e) }
    w = TkLabel.new( root, 'image'=>cphoto )
    w.grid( 'row'=>1, 'column'=>0 )
    w.bind( "Button1-Motion" ){|e,i,o,s| i||i=cphoto; o||o='coronal'; s||s=self; s.Motion( e, i, o ) }
    w.bind( "Button-3" ){|e| self.DoPopup(e) }
    w = TkLabel.new( root, 'image'=>sphoto )
    w.grid( 'row'=>0, 'column'=>1 )
    w.bind( "Button1-Motion" ){|e,i,o.s| i||i=sphoto; o||o='sagittal'; s||s=self; s.Motion( e, i, o ) }
    w.bind( "Button-3" ){|e| self.DoPopup(e) }
    w = @WindowWidget = TkScale.new( root, 'label'=>'Window', 'orient'=>'horizontal', 'from'=>1, 'to'=>(h-l)/2, 'command'=>setWindowLevel )
    w = @LevelWidget = TkScale.new( root, 'label'=>'Level', 'orient'=>'horizontal', 'from'=>l, 'to'=>h, 'command'=>setWindowLevel )
    @WindowWidget.grid( 'row'=>2, 'columnspan'=>2, 'sticky'=>'ew' )
    @LevelWidget.grid( 'row'=>3, 'columnspan'=>2, 'sticky'=>'ew' );
    @WindowWidget.set( 1370 );
    @LevelWidget.set( 1268 );

    w = @LabelWidget = TkLabel.new( root, 'bd'=>2, 'relief'=>'raised' )
    w.grid( 'row'=>4, 'columnspan'=>2, 'sticky'=>'ew' )
    w.configure( 'text'=>"Use the right mouse button to change data type" )
  end


  def DoPopup( event )
    @popup.post( event.x_root, event.y_root )
  end

  def Motion( event, image, orientation )
    w = image.width
    h = image.height
    if orientation == 'transverse'
      @Position[0] = event.x
      @Position[1] = h - event.y - 1
    end
    if orientation == 'coronal'
      @Position[0] = event.x
      @Position[2] = event.y
    end
    if orientation == 'sagittal'
      @Position[1] = w - event.x - 1
      @Position[2] = event.y
    end
    x, y, z = @Posistion
    @LabelWidget.configure( 'text'=>"Position: #{"%d"%x}, #{"%d"%y}, #{"%d"%z}" )
    self.SetImages
  end

  def SetImages
    window = @WindowWidget.get
    level = @LevelWidget.get
    image = @resample.GetOutput 
    @tphoto.PutImageSlice( image,
                           @Position[2],
                           'orientation'=>'transverse',
                           'window'=>window,
                           'level'=>level )
    @sphoto.PutImageSlice( image,
                           @Position[0],
                           'orientation'=>'sagittal',
                           'window'=>window,
                           'level'=>level )
    @cphoto.PutImageSlice( image,
                           @Position[1],
                           'orientation'=>'coronal',
                           'window'=>window,
                           'level'=>level )
  end

end



if $0 == __FILE__
  S = SampleViewer.new
  Tk.mainloop
end
