#!/usr/bin/env ruby

# This example demonstrates how to use the vtkPointWidget to probe a
# dataset && then color the probed point (represented as a cone) as
# per the probed value && orient it as per the vector.

require 'vtk'
require 'vtk/util'
VTK_DATA_ROOT = vtkGetDataRoot

# Start by loading some data.
pl3d = Vtk::PLOT3DReader.new
pl3d.SetXYZFileName(VTK_DATA_ROOT + "/Data/combxyz.bin")
pl3d.SetQFileName(VTK_DATA_ROOT + "/Data/combq.bin")
pl3d.SetScalarFunctionNumber(100)
pl3d.SetVectorFunctionNumber(202)
pl3d.Update

# The plane widget is used probe the dataset.
pointWidget = Vtk::PointWidget.new
pointWidget.SetInput(pl3d.GetOutput)
pointWidget.AllOff
pointWidget.PlaceWidget
point = Vtk::PolyData.new
pointWidget.GetPolyData(point)

probe = Vtk::ProbeFilter.new
probe.SetInput(point)
probe.SetSource(pl3d.GetOutput)

# create glyph
cone = Vtk::ConeSource.new
cone.SetResolution(16)
glyph = Vtk::Glyph3D.new
glyph.SetInputConnection(probe.GetOutputPort)
glyph.SetSource(cone.GetOutput)
glyph.SetVectorModeToUseVector
glyph.SetScaleModeToDataScalingOff
glyph.SetScaleFactor(pl3d.GetOutput.GetLength*0.1)
glyphMapper = Vtk::PolyDataMapper.new
glyphMapper.SetInputConnection(glyph.GetOutputPort)
glyphActor = Vtk::Actor.new
glyphActor.SetMapper(glyphMapper)
glyphActor.VisibilityOff

# An outline is shown for context.
outline = Vtk::StructuredGridOutlineFilter.new
outline.SetInputConnection(pl3d.GetOutputPort)
outlineMapper = Vtk::PolyDataMapper.new
outlineMapper.SetInputConnection(outline.GetOutputPort)
outlineActor = Vtk::Actor.new
outlineActor.SetMapper(outlineMapper)

# Create the RenderWindow, Renderer && both Actors
ren = Vtk::Renderer.new
renWin = Vtk::RenderWindow.new
renWin.AddRenderer(ren)
iren = Vtk::RenderWindowInteractor.new
iren.SetRenderWindow(renWin)

# Actually set the color && orientation of the probe.
BeginInteraction = Proc.new{|obj, event|
    obj.GetPolyData(point)
    glyphActor.VisibilityOn 
}

ProbeData = Proc.new{|obj, event|
    obj.GetPolyData(point) 
}

# Associate the line widget with the interactor
pointWidget.SetInteractor(iren)
pointWidget.AddObserver("EnableEvent", BeginInteraction)
pointWidget.AddObserver("StartInteractionEvent", BeginInteraction)
pointWidget.AddObserver("InteractionEvent", ProbeData)

# Add the actors to the renderer, set the background && size
ren.AddActor(outlineActor)
ren.AddActor(glyphActor)

ren.SetBackground(1, 1, 1)
renWin.SetSize(300, 300)
ren.SetBackground(0.1, 0.2, 0.4)

cam1 = ren.GetActiveCamera
cam1.SetClippingRange(3.95297, 50)
cam1.SetFocalPoint(9.71821, 0.458166, 29.3999)
cam1.SetPosition(2.7439, -37.3196, 38.7167)
cam1.SetViewUp(-0.16123, 0.264271, 0.950876)

iren.Initialize
renWin.Render
iren.Start
