#!/usr/bin/env ruby

# This example demonstrates how to use the vtkPlaneWidget to probe
# a dataset && then generate contours on the probed data.

require 'vtk'
require 'vtk/util'
VTK_DATA_ROOT = vtkGetDataRoot

# Start by loading some data.
pl3d = Vtk::PLOT3DReader.new
pl3d.SetXYZFileName(VTK_DATA_ROOT + "/Data/combxyz.bin")
pl3d.SetQFileName(VTK_DATA_ROOT + "/Data/combq.bin")
pl3d.SetScalarFunctionNumber(100)
pl3d.SetVectorFunctionNumber(202)
pl3d.Update

# The plane widget is used probe the dataset.
planeWidget = Vtk::PlaneWidget.new
planeWidget.SetInput(pl3d.GetOutput)
planeWidget.NormalToXAxisOn
planeWidget.SetResolution(20)
planeWidget.SetRepresentationToOutline
planeWidget.PlaceWidget
plane = Vtk::PolyData.new
planeWidget.GetPolyData(plane)

probe = Vtk::ProbeFilter.new
probe.SetInput(plane)
probe.SetSource(pl3d.GetOutput)

contourMapper = Vtk::PolyDataMapper.new
contourMapper.SetInputConnection(probe.GetOutputPort)
contourMapper.SetScalarRange(pl3d.GetOutput.GetScalarRange)
contourActor = Vtk::Actor.new
contourActor.SetMapper(contourMapper)
contourActor.VisibilityOff

# An outline is shown for context.
outline = Vtk::StructuredGridOutlineFilter.new
outline.SetInputConnection(pl3d.GetOutputPort)
outlineMapper = Vtk::PolyDataMapper.new
outlineMapper.SetInputConnection(outline.GetOutputPort)
outlineActor = Vtk::Actor.new
outlineActor.SetMapper(outlineMapper)

# Create the RenderWindow, Renderer && both Actors
ren = Vtk::Renderer.new
renWin = Vtk::RenderWindow.new
renWin.AddRenderer(ren)
iren = Vtk::RenderWindowInteractor.new
iren.SetRenderWindow(renWin)

# Actually generate contour lines.
BeginInteraction = Proc.new{|obj, event|
    global plane, contourActor
    obj.GetPolyData(plane)
    contourActor.VisibilityOn
}

ProbeData = Proc.new{|obj, event|
    global plane
    obj.GetPolyData(plane)
}


# Associate the widget with the interactor
planeWidget.SetInteractor(iren)
# Handle the events.
planeWidget.AddObserver("EnableEvent", BeginInteraction)
planeWidget.AddObserver("StartInteractionEvent", BeginInteraction)
planeWidget.AddObserver("InteractionEvent", ProbeData)

# Add the actors to the renderer, set the background && size
ren.AddActor(outlineActor)
ren.AddActor(contourActor)

ren.SetBackground(1, 1, 1)
renWin.SetSize(300, 300)
ren.SetBackground(0.1, 0.2, 0.4)

cam1 = ren.GetActiveCamera
cam1.SetClippingRange(3.95297, 50)
cam1.SetFocalPoint(9.71821, 0.458166, 29.3999)
cam1.SetPosition(2.7439, -37.3196, 38.7167)
cam1.SetViewUp(-0.16123, 0.264271, 0.950876)

iren.Initialize
renWin.Render
iren.Start
