#!/usr/bin/env ruby

# This example demonstrates the use of vtkSTLReader to load data into
# VTK from a file.  This example also uses vtkLODActor which changes
# its graphical representation of the data to maintain interactive
# performance.

require 'vtk'
require 'vtk/util'
VTK_DATA_ROOT = vtkGetDataRoot

# Create the reader && read a data file.  Connect the mapper &&
# actor.
sr = Vtk::STLReader.new
sr.SetFileName(VTK_DATA_ROOT + "/Data/42400-IDGH.stl")

stlMapper = Vtk::PolyDataMapper.new
stlMapper.SetInputConnection(sr.GetOutputPort)

stlActor = Vtk::LODActor.new
stlActor.SetMapper(stlMapper)

# Create the Renderer, RenderWindow, && RenderWindowInteractor
ren = Vtk::Renderer.new
renWin = Vtk::RenderWindow.new
renWin.AddRenderer(ren)
iren = Vtk::RenderWindowInteractor.new
iren.SetRenderWindow(renWin)

# Add the actors to the render; set the background && size
ren.AddActor(stlActor)
ren.SetBackground(0.1, 0.2, 0.4)
renWin.SetSize(500, 500)

# Zoom in closer
ren.ResetCamera
cam1 = ren.GetActiveCamera
cam1.Zoom(1.4)

iren.Initialize
renWin.Render
iren.Start
