#!/usr/bin/env ruby

# This example shows how to use Delaunay3D with alpha shapes.

require 'vtk'

# The points to be triangulated are generated randomly in the unit
# cube located at the origin. The points are then associated with a
# vtkPolyData.
math = Vtk::Math
points = Vtk::Points.new
for i in 0...25
    points.InsertPoint(i, math.Random(0, 1), math.Random(0, 1),
                       math.Random(0, 1))
end

profile = Vtk::PolyData.new
profile.SetPoints(points)

# Delaunay3D is used to triangulate the points. The Tolerance is the
# distance that nearly coincident points are merged
# together. (Delaunay does better if points are well spaced.) The
# alpha value is the radius of circumcircles, circumspheres. Any mesh
# entity whose circumcircle is smaller than this value is output.
delny = Vtk::Delaunay3D.new
delny.SetInput(profile)
delny.SetTolerance(0.01)
delny.SetAlpha(0.2)
delny.BoundingTriangulationOff

# Shrink the result to help see it better.
shrink = Vtk::ShrinkFilter.new
shrink.SetInputConnection(delny.GetOutputPort)
shrink.SetShrinkFactor(0.9)

map = Vtk::DataSetMapper.new
map.SetInputConnection(shrink.GetOutputPort)

triangulation = Vtk::Actor.new
triangulation.SetMapper(map)
triangulation.GetProperty.SetColor(1, 0, 0)

# Create graphics stuff
ren = Vtk::Renderer.new
renWin = Vtk::RenderWindow.new
renWin.AddRenderer(ren)
iren = Vtk::RenderWindowInteractor.new
iren.SetRenderWindow(renWin)

# Add the actors to the renderer, set the background and size
ren.AddActor(triangulation)
ren.SetBackground(1, 1, 1)
renWin.SetSize(250, 250)
renWin.Render

cam1 = ren.GetActiveCamera
cam1.Zoom(1.5)

iren.Initialize
renWin.Render
iren.Start
