#!/usr/bin/env ruby

# This example demonstrates how to combine a "geometric" implicit
# function with noise at different frequencies to produce the
# appearance of a landscape.

require 'vtk'

plane = Vtk::Plane.new

p1 = Vtk::PerlinNoise.new
p1.SetFrequency(1, 1, 0)

p2 = Vtk::PerlinNoise.new
p2.SetFrequency(3, 5, 0)
p2.SetPhase(0.5, 0.5, 0)

p3 = Vtk::PerlinNoise.new
p3.SetFrequency(16, 16, 0)

sum = Vtk::ImplicitSum.new
sum.SetNormalizeByWeight(1)
sum.AddFunction(plane)
sum.AddFunction(p1, 0.2)
sum.AddFunction(p2, 0.1)
sum.AddFunction(p3, 0.02)

sample = Vtk::SampleFunction.new
sample.SetImplicitFunction(sum)
sample.SetSampleDimensions(65, 65, 20)
sample.SetModelBounds(-1, 1, -1, 1, -0.5, 0.5)
sample.ComputeNormalsOff
surface = Vtk::ContourFilter.new
surface.SetInputConnection(sample.GetOutputPort)
surface.SetValue(0, 0.0)

smooth = Vtk::PolyDataNormals.new
smooth.SetInputConnection(surface.GetOutputPort)
smooth.SetFeatureAngle(90)

mapper = Vtk::PolyDataMapper.new
mapper.SetInputConnection(smooth.GetOutputPort)
mapper.ScalarVisibilityOff
actor = Vtk::Actor.new
actor.SetMapper(mapper)
actor.GetProperty.SetColor(0.4, 0.2, 0.1)

# Create the renderer etc.
ren = Vtk::Renderer.new
renWin = Vtk::RenderWindow.new
renWin.AddRenderer(ren)
iren = Vtk::RenderWindowInteractor.new
iren.SetRenderWindow(renWin)

# Add the actors to the renderer, set the background and size
ren.AddActor(actor)
ren.SetBackground(1, 1, 1)
renWin.SetSize(500, 500)
ren.ResetCamera
ren.GetActiveCamera.Elevation(-45)
ren.GetActiveCamera.Azimuth(10)
ren.GetActiveCamera.Dolly(1.35)
ren.ResetCameraClippingRange

iren.Initialize
renWin.Render
iren.Start
