#!/usr/bin/env ruby

# A game with VTK && Tk. :)

require 'tk'
require 'vtk'
require 'vtk/tk'

# Create the pipeline
@puzzle = Vtk::SpherePuzzle.new
mapper = Vtk::PolyDataMapper.new
mapper.SetInputConnection(@puzzle.GetOutputPort)
actor = Vtk::Actor.new
actor.SetMapper(mapper)

arrows = Vtk::SpherePuzzleArrows.new
mapper2 = Vtk::PolyDataMapper.new
mapper2.SetInputConnection(arrows.GetOutputPort)
actor2 = Vtk::Actor.new
actor2.SetMapper(mapper2)

@renWin = Vtk::RenderWindow.new
@ren = Vtk::Renderer.new
@renWin.AddRenderer(@ren)

# Add the actors to the renderer, set the background && size
@ren.AddActor(actor)
@ren.AddActor(actor2)
@ren.SetBackground(0.1, 0.2, 0.4)

@ren.ResetCamera
cam = @ren.GetActiveCamera
cam.Elevation(-40)


## Generate the GUI
@root = TkRoot.new{
  title "Sphere Puzzle"
}

# Define a quit method that exits cleanly.
quit = Proc.new{
  exit
}

# Create the toplevel window
@root.protocol("WM_DELETE_WINDOW", quit)

# Create some frames
f1 = TkFrame.new(@root)
f2 = TkFrame.new(@root)

f1.pack("side"=>"top", "anchor"=>"n", "expand"=>1, "fill"=>"both")
f2.pack("side"=>"bottom", "anchor"=>"s", "expand"=>"t", "fill"=>"x")

# Create the Tk render widget, && bind the events
@iren = Vtk::TkRenderWindowInteractor.new(f1, 'width'=>400, 'height'=>400, 'rw'=>@renWin)
@iren.pack("expand"=>"t", "fill"=>"both")

reset = Proc.new{
  @puzzle.Reset
  @renWin.Render
}

# Display some information
l1 = TkLabel.new(f2, 'text'=>"Position cursor over the rotation plane.")
l2 = TkLabel.new(f2, 'text'=>"Moving pieces will be highlighted.")
l3 = TkLabel.new(f2, 'text'=>"Press 'm' to make a move.")
reset = TkButton.new(f2, 'text'=>"Reset", 'command'=>reset)

b1 = TkButton.new(f2, "text"=>"Quit", "command"=>quit)

for i in [l1, l2, l3, reset, b1]
    i.pack("side"=>"top", "expand"=>"t", "fill"=>"x")
end

# Done with the GUI.  Create callback functions.

@in_piece_rotation = false
@lastVal = nil

# Highlight pieces
MotionCallback = Proc.new{|obj, event|
  if @in_piece_rotation
    return 
  end

  istyle = @iren.GetInteractorStyle.GetCurrentStyle

  # Return if the user is performing interaction
  if istyle.GetState != 0
    return
  end

  # Get mouse position
  pos = @iren.GetEventPosition
  x, y = pos

  # Get world point
  @ren.SetDisplayPoint(x, y, @ren.GetZ(x, y))
  @ren.DisplayToWorld
  pt = @ren.GetWorldPoint
  val = @puzzle.SetPoint(pt[0], pt[1], pt[2])

  if ( !@lastVal) || val != @lastVal
    @renWin.Render
    @lastVal = val
  end
}

# Rotate the puzzle
CharCallback = Proc.new{|obj, event|
  keycode = @iren.GetKeyCode
  if keycode != "m" && keycode != "M"
    return
  end

  pos = @iren.GetEventPosition
  ButtonCallback(pos[0], pos[1])
}

def ButtonCallback(x, y)
  if @in_piece_rotation
    return
  end
  @in_piece_rotation = true

  # Get world point
  @ren.SetDisplayPoint(x, y, @ren.GetZ(x,y))
  @ren.DisplayToWorld
  pt = @ren.GetWorldPoint
  x, y, z = pt[0..2]

  for ii in 0..10
    i = ii*10
    @puzzle.SetPoint(x, y, z)
    @puzzle.MovePoint(i)
    @renWin.Render
    @root.update
  end

  @in_piece_rotation = false
end

@root.update

# Modify some bindings, use the interactor style 'switch'
istyle = Vtk::InteractorStyleSwitch.new

@iren.SetInteractorStyle(istyle)
istyle.SetCurrentStyleToTrackballCamera

@iren.AddObserver("MouseMoveEvent", MotionCallback)
@iren.AddObserver("CharEvent", CharCallback)

# Shuffle the puzzle
ButtonCallback(218, 195)
ButtonCallback(261, 128)
ButtonCallback(213, 107)
ButtonCallback(203, 162)
ButtonCallback(134, 186)

@iren.Initialize
@renWin.Render
@iren.Start

@root.mainloop
