#!/usr/bin/env ruby

# This example demonstrates how to use implicit modelling.

require 'vtk'
require 'vtk/util'
VTK_DATA_ROOT = vtkGetDataRoot

# Create lines which serve as the "seed" geometry. The lines spell the
# word "hello".
reader = Vtk::PolyDataReader.new
reader.SetFileName(VTK_DATA_ROOT + "/Data/hello.vtk")
lineMapper = Vtk::PolyDataMapper.new
lineMapper.SetInputConnection(reader.GetOutputPort)
lineActor = Vtk::Actor.new
lineActor.SetMapper(lineMapper)
lineActor.GetProperty.SetColor(Vtk::Colors::Red)

# Create implicit model with vtkImplicitModeller. This computes a
# scalar field which is the distance from the generating geometry. The
# contour filter then extracts the geoemtry at the distance value 0.25
# from the generating geometry.
imp = Vtk::ImplicitModeller.new
imp.SetInputConnection(reader.GetOutputPort)
imp.SetSampleDimensions(110, 40, 20)
imp.SetMaximumDistance(0.25)
imp.SetModelBounds(-1.0, 10.0, -1.0, 3.0, -1.0, 1.0)
contour = Vtk::ContourFilter.new
contour.SetInputConnection(imp.GetOutputPort)
contour.SetValue(0, 0.25)
impMapper = Vtk::PolyDataMapper.new
impMapper.SetInputConnection(contour.GetOutputPort)
impMapper.ScalarVisibilityOff
impActor = Vtk::Actor.new
impActor.SetMapper(impMapper)
impActor.GetProperty.SetColor(Vtk::Colors::Peacock)
impActor.GetProperty.SetOpacity(0.5)

# Create the usual graphics stuff.
# Create the RenderWindow, Renderer and both Actors
ren = Vtk::Renderer.new
renWin = Vtk::RenderWindow.new
renWin.AddRenderer(ren)
iren = Vtk::RenderWindowInteractor.new
iren.SetRenderWindow(renWin)

# Add the actors to the renderer, set the background and size
ren.AddActor(lineActor)
ren.AddActor(impActor)
ren.SetBackground(1, 1, 1)
renWin.SetSize(600, 250)

camera = Vtk::Camera.new
camera.SetClippingRange(1.81325, 90.6627)
camera.SetFocalPoint(4.5, 1, 0)
camera.SetPosition(4.5, 1.0, 6.73257)
camera.SetViewUp(0, 1, 0)
camera.Zoom(0.8)
ren.SetActiveCamera(camera)

iren.Initialize
renWin.Render
iren.Start
