#!/usr/bin/env ruby

# This simple example shows how to do basic texture mapping.

require 'vtk'
require 'vtk/util'
VTK_DATA_ROOT = vtkGetDataRoot

# Load in the texture map. A texture is any unsigned char image. If it
# is not of this type, you will have to map it through a lookup table
# or by using vtkImageShiftScale.
bmpReader = Vtk::BMPReader.new
bmpReader.SetFileName(VTK_DATA_ROOT + "/Data/masonry.bmp")
atext = Vtk::Texture.new
atext.SetInputConnection(bmpReader.GetOutputPort)
atext.InterpolateOn

# Create a plane source and actor. The vtkPlanesSource generates
# texture coordinates.
plane = Vtk::PlaneSource.new
planeMapper = Vtk::PolyDataMapper.new
planeMapper.SetInputConnection(plane.GetOutputPort)
planeActor = Vtk::Actor.new
planeActor.SetMapper(planeMapper)
planeActor.SetTexture(atext)

# Create the RenderWindow, Renderer and both Actors
ren = Vtk::Renderer.new
renWin = Vtk::RenderWindow.new
renWin.AddRenderer(ren)
iren = Vtk::RenderWindowInteractor.new
iren.SetRenderWindow(renWin)

# Add the actors to the renderer, set the background and size
ren.AddActor(planeActor)
ren.SetBackground(0.1, 0.2, 0.4)
renWin.SetSize(500, 500)

ren.ResetCamera
cam1 = ren.GetActiveCamera
cam1.Elevation(-30)
cam1.Roll(-20)
ren.ResetCameraClippingRange

iren.Initialize
renWin.Render
iren.Start
