#!/usr/bin/env ruby
#
#
# This example shows how to add an observer to a Ruby program. It extends
# the Step1/Ruby/Cone.rb Ruby example (see that example for information on
# the basic setup). 
#
# VTK uses a command/observer design pattern. That is, observers watch for
# particular events that any vtkObject (or subclass) may invoke on
# itself. For example, the vtkRenderer invokes a "StartEvent" as it begins
# to render. Here we add an observer that invokes a command when this event
# is observed.
#

require 'vtk'

#
# define the callback
#
myCallback = Proc.new {|obj,string|
    print "Starting a render\n"
}


#
# create the basic pipeline as in Step1
#
cone = Vtk::ConeSource.new
cone.SetHeight( 3.0 )
cone.SetRadius( 1.0 )
cone.SetResolution( 10 )

coneMapper = Vtk::PolyDataMapper.new
coneMapper.SetInputConnection( cone.GetOutputPort )
coneActor = Vtk::Actor.new
coneActor.SetMapper( coneMapper )

ren1= Vtk::Renderer.new
ren1.AddActor( coneActor )
ren1.SetBackground( 0.1, 0.2, 0.4 )

#
# Add the observer here
#
ren1.AddObserver("StartEvent", myCallback)

renWin = Vtk::RenderWindow.new
renWin.AddRenderer( ren1 )
renWin.SetSize( 300, 300 )

#
# now we loop over 360 degreeees and render the cone each time
#
for i in 0...360
  sleep 0.03
  renWin.Render
  ren1.GetActiveCamera.Azimuth( 1 )
end
  
