#!/usr/bin/env ruby

# This example shows how to extract a piece of a dataset using an
# implicit function. In this case the implicit function is formed by
# the boolean combination of two ellipsoids.

require 'vtk'

# Here we create two ellipsoidal implicit functions and boolean them
# together tto form a "cross" shaped implicit function.
quadric = Vtk::Quadric.new
quadric.SetCoefficients(0.5, 1, 0.2, 0, 0.1, 0, 0, 0.2, 0, 0)
sample = Vtk::SampleFunction.new
sample.SetSampleDimensions(50, 50, 50)
sample.SetImplicitFunction(quadric)
sample.ComputeNormalsOff
trans = Vtk::Transform.new
trans.Scale(1, 0.5, 0.333)
sphere = Vtk::Sphere.new
sphere.SetRadius(0.25)
sphere.SetTransform(trans)
trans2 = Vtk::Transform.new
trans2.Scale(0.25, 0.5, 1.0)
sphere2 = Vtk::Sphere.new
sphere2.SetRadius(0.25)
sphere2.SetTransform(trans2)
union = Vtk::ImplicitBoolean.new
union.AddFunction(sphere)
union.AddFunction(sphere2)
union.SetOperationType(0) #union

# Here is where it gets interesting. The implicit function is used to
# extract those cells completely inside the function. They are then 
# shrunk to help show what was extracted.
extract = Vtk::ExtractGeometry.new
extract.SetInputConnection(sample.GetOutputPort)
extract.SetImplicitFunction(union)
shrink = Vtk::ShrinkFilter.new
shrink.SetInputConnection(extract.GetOutputPort)
shrink.SetShrinkFactor(0.5)
dataMapper = Vtk::DataSetMapper.new
dataMapper.SetInputConnection(shrink.GetOutputPort)
dataActor = Vtk::Actor.new
dataActor.SetMapper(dataMapper)

# The outline gives context to the original data.
outline = Vtk::OutlineFilter.new
outline.SetInputConnection(sample.GetOutputPort)
outlineMapper = Vtk::PolyDataMapper.new
outlineMapper.SetInputConnection(outline.GetOutputPort)
outlineActor = Vtk::Actor.new
outlineActor.SetMapper(outlineMapper)
outlineProp = outlineActor.GetProperty
outlineProp.SetColor(0, 0, 0)

# The usual rendering stuff is created.
ren = Vtk::Renderer.new
renWin = Vtk::RenderWindow.new
renWin.AddRenderer(ren)
iren = Vtk::RenderWindowInteractor.new
iren.SetRenderWindow(renWin)

# Add the actors to the renderer, set the background and size
ren.AddActor(outlineActor)
ren.AddActor(dataActor)
ren.SetBackground(1, 1, 1)
renWin.SetSize(500, 500)
ren.ResetCamera
ren.GetActiveCamera.Zoom(1.5)

iren.Initialize
renWin.Render
iren.Start
