#!/usr/bin/env ruby

# This example demonstrates the subsampling of a structured grid.

require 'vtk'
require 'vtk/util'
VTK_DATA_ROOT = vtkGetDataRoot

# Read some structured data.
pl3d = Vtk::PLOT3DReader.new
pl3d.SetXYZFileName(VTK_DATA_ROOT + "/Data/combxyz.bin")
pl3d.SetQFileName(VTK_DATA_ROOT + "/Data/combq.bin")
pl3d.SetScalarFunctionNumber(100)
pl3d.SetVectorFunctionNumber(202)
pl3d.Update

# Here we subsample the grid. The SetVOI method requires six values
# specifying (imin,imax, jmin,jmax, kmin,kmax) extents. In this
# example we extracting a plane. Note that the VOI is clamped to zero
# (min) and the maximum i-j-k value; that way we can use the
# -1000,1000 specification and be sure the values are clamped. The
# SampleRate specifies that we take every point in the i-direction;
# every other point in the j-direction; and every third point in the
# k-direction. IncludeBoundaryOn makes sure that we get the boundary
# points even if the SampleRate does not coincident with the boundary.
extract = Vtk::ExtractGrid.new
extract.SetInputConnection(pl3d.GetOutputPort)
extract.SetVOI(30, 30, -1000, 1000, -1000, 1000)
extract.SetSampleRate(1, 2, 3)
extract.IncludeBoundaryOn
mapper = Vtk::DataSetMapper.new
mapper.SetInputConnection(extract.GetOutputPort)
mapper.SetScalarRange(0.18, 0.7)
actor = Vtk::Actor.new
actor.SetMapper(mapper)

outline = Vtk::StructuredGridOutlineFilter.new
outline.SetInputConnection(pl3d.GetOutputPort)
outlineMapper = Vtk::PolyDataMapper.new
outlineMapper.SetInputConnection(outline.GetOutputPort)
outlineActor = Vtk::Actor.new
outlineActor.SetMapper(outlineMapper)
outlineActor.GetProperty.SetColor(0, 0, 0)

# Add the usual rendering stuff.
ren = Vtk::Renderer.new
renWin = Vtk::RenderWindow.new
renWin.AddRenderer(ren)
iren = Vtk::RenderWindowInteractor.new
iren.SetRenderWindow(renWin)

# Add the actors to the renderer, set the background and size
ren.AddActor(outlineActor)
ren.AddActor(actor)

ren.SetBackground(1, 1, 1)
renWin.SetSize(300, 180)

cam1 = ren.GetActiveCamera
cam1.SetClippingRange(2.64586, 47.905)
cam1.SetFocalPoint(8.931, 0.358127, 31.3526)
cam1.SetPosition(29.7111, -0.688615, 37.1495)
cam1.SetViewUp(-0.268328, 0.00801595, 0.963294)

iren.Initialize
renWin.Render
iren.Start
