#!/usr/bin/env ruby

# This example shows how to combine data from both the imaging and
# graphics pipelines. The vtkMergeFilter is used to merge the data
# from each together.

require 'vtk'
require 'vtk/util'
VTK_DATA_ROOT = vtkGetDataRoot

# Read in an image and compute a luminance value. The image is
# extracted as a set of polygons (vtkImageDataGeometryFilter). We then
# will warp the plane using the scalar (luminance) values.
reader = Vtk::BMPReader.new
reader.SetFileName(VTK_DATA_ROOT + "/Data/masonry.bmp")
luminance = Vtk::ImageLuminance.new
luminance.SetInputConnection(reader.GetOutputPort)
geometry = Vtk::ImageDataGeometryFilter.new
geometry.SetInputConnection(luminance.GetOutputPort)
warp = Vtk::WarpScalar.new
warp.SetInputConnection(geometry.GetOutputPort)
warp.SetScaleFactor(-0.1)

# Use vtkMergeFilter to combine the original image with the warped
# geometry.
merge = Vtk::MergeFilter.new
merge.SetGeometry(warp.GetOutput)
merge.SetScalars(reader.GetOutput)
mapper = Vtk::DataSetMapper.new
mapper.SetInputConnection(merge.GetOutputPort)
mapper.SetScalarRange(0, 255)
mapper.ImmediateModeRenderingOff
actor = Vtk::Actor.new
actor.SetMapper(mapper)

# Create renderer stuff
ren = Vtk::Renderer.new
renWin = Vtk::RenderWindow.new
renWin.AddRenderer(ren)
iren = Vtk::RenderWindowInteractor.new
iren.SetRenderWindow(renWin)

# Add the actors to the renderer, set the background and size
ren.AddActor(actor)
ren.ResetCamera
ren.GetActiveCamera.Azimuth(20)
ren.GetActiveCamera.Elevation(30)
ren.SetBackground(0.1, 0.2, 0.4)
ren.ResetCameraClippingRange

renWin.SetSize(250, 250)

cam1 = ren.GetActiveCamera
cam1.Zoom(1.4)

iren.Initialize
renWin.Render
iren.Start
