#!/usr/bin/env ruby

# This shows how to probe a dataset with a plane. The probed data is
# then contoured.

require 'vtk'
require 'vtk/util'
VTK_DATA_ROOT = vtkGetDataRoot

# Read data.
pl3d = Vtk::PLOT3DReader.new
pl3d.SetXYZFileName(VTK_DATA_ROOT + "/Data/combxyz.bin")
pl3d.SetQFileName(VTK_DATA_ROOT + "/Data/combq.bin")
pl3d.SetScalarFunctionNumber(100)
pl3d.SetVectorFunctionNumber(202)
pl3d.Update

# We create three planes and position them in the correct position
# using transform filters. They are then appended together and used as
# a probe.
plane = Vtk::PlaneSource.new
plane.SetResolution(50, 50)
transP1 = Vtk::Transform.new
transP1.Translate(3.7, 0.0, 28.37)
transP1.Scale(5, 5, 5)
transP1.RotateY(90)
tpd1 = Vtk::TransformPolyDataFilter.new
tpd1.SetInputConnection(plane.GetOutputPort)
tpd1.SetTransform(transP1)
outTpd1 = Vtk::OutlineFilter.new
outTpd1.SetInputConnection(tpd1.GetOutputPort)
mapTpd1 = Vtk::PolyDataMapper.new
mapTpd1.SetInputConnection(outTpd1.GetOutputPort)
tpd1Actor = Vtk::Actor.new
tpd1Actor.SetMapper(mapTpd1)
tpd1Actor.GetProperty.SetColor(0, 0, 0)

transP2 = Vtk::Transform.new
transP2.Translate(9.2, 0.0, 31.20)
transP2.Scale(5, 5, 5)
transP2.RotateY(90)
tpd2 = Vtk::TransformPolyDataFilter.new
tpd2.SetInputConnection(plane.GetOutputPort)
tpd2.SetTransform(transP2)
outTpd2 = Vtk::OutlineFilter.new
outTpd2.SetInputConnection(tpd2.GetOutputPort)
mapTpd2 = Vtk::PolyDataMapper.new
mapTpd2.SetInputConnection(outTpd2.GetOutputPort)
tpd2Actor = Vtk::Actor.new
tpd2Actor.SetMapper(mapTpd2)
tpd2Actor.GetProperty.SetColor(0, 0, 0)

transP3 = Vtk::Transform.new
transP3.Translate(13.27, 0.0, 33.30)
transP3.Scale(5, 5, 5)
transP3.RotateY(90)
tpd3 = Vtk::TransformPolyDataFilter.new
tpd3.SetInputConnection(plane.GetOutputPort)
tpd3.SetTransform(transP3)
outTpd3 = Vtk::OutlineFilter.new
outTpd3.SetInputConnection(tpd3.GetOutputPort)
mapTpd3 = Vtk::PolyDataMapper.new
mapTpd3.SetInputConnection(outTpd3.GetOutputPort)
tpd3Actor = Vtk::Actor.new
tpd3Actor.SetMapper(mapTpd3)
tpd3Actor.GetProperty.SetColor(0, 0, 0)

appendF = Vtk::AppendPolyData.new
appendF.AddInput(tpd1.GetOutput)
appendF.AddInput(tpd2.GetOutput)
appendF.AddInput(tpd3.GetOutput)

# The vtkProbeFilter takes two inputs. One is a dataset to use as the
# probe geometry (SetInput); the other is the data to probe
# (SetSource). The output dataset structure (geometry and topology) of
# the probe is the same as the structure of the input. The probing
# process generates new data values resampled from the source.
probe = Vtk::ProbeFilter.new
probe.SetInputConnection(appendF.GetOutputPort)
probe.SetSource(pl3d.GetOutput)

contour = Vtk::ContourFilter.new
contour.SetInputConnection(probe.GetOutputPort)
contour.GenerateValues(50, pl3d.GetOutput.GetScalarRange)
contourMapper = Vtk::PolyDataMapper.new
contourMapper.SetInputConnection(contour.GetOutputPort)
contourMapper.SetScalarRange(pl3d.GetOutput.GetScalarRange)
planeActor = Vtk::Actor.new
planeActor.SetMapper(contourMapper)

outline = Vtk::StructuredGridOutlineFilter.new
outline.SetInputConnection(pl3d.GetOutputPort)
outlineMapper = Vtk::PolyDataMapper.new
outlineMapper.SetInputConnection(outline.GetOutputPort)
outlineActor = Vtk::Actor.new
outlineActor.SetMapper(outlineMapper)
outlineActor.GetProperty.SetColor(0, 0, 0)

# Create the RenderWindow, Renderer and both Actors
ren = Vtk::Renderer.new
renWin = Vtk::RenderWindow.new
renWin.AddRenderer(ren)
iren = Vtk::RenderWindowInteractor.new
iren.SetRenderWindow(renWin)

ren.AddActor(outlineActor)
ren.AddActor(planeActor)
ren.AddActor(tpd1Actor)
ren.AddActor(tpd2Actor)
ren.AddActor(tpd3Actor)
ren.SetBackground(1, 1, 1)
renWin.SetSize(400, 400)

ren.ResetCamera
cam1 = ren.GetActiveCamera
cam1.SetClippingRange(3.95297, 50)
cam1.SetFocalPoint(8.88908, 0.595038, 29.3342)
cam1.SetPosition(-12.3332, 31.7479, 41.2387)
cam1.SetViewUp(0.060772, -0.319905, 0.945498)

iren.Initialize
renWin.Render
iren.Start
