#!/usr/bin/env ruby

# This example demonstrates the generation of a streamsurface.

require 'vtk'
require 'vtk/util'
VTK_DATA_ROOT = vtkGetDataRoot

# Read the data and specify which scalars and vectors to read.
pl3d = Vtk::PLOT3DReader.new
pl3d.SetXYZFileName(VTK_DATA_ROOT + "/Data/combxyz.bin")
pl3d.SetQFileName(VTK_DATA_ROOT + "/Data/combq.bin")
pl3d.SetScalarFunctionNumber(100)
pl3d.SetVectorFunctionNumber(202)
pl3d.Update

# We use a rake to generate a series of streamline starting points
# scattered along a line. Each point will generate a streamline. These
# streamlines are then fed to the vtkRuledSurfaceFilter which stitches
# the lines together to form a surface.
rake = Vtk::LineSource.new
rake.SetPoint1(15, -5, 32)
rake.SetPoint2(15, 5, 32)
rake.SetResolution(21)
rakeMapper = Vtk::PolyDataMapper.new
rakeMapper.SetInputConnection(rake.GetOutputPort)
rakeActor = Vtk::Actor.new
rakeActor.SetMapper(rakeMapper)

integ = Vtk::RungeKutta4.new
sl = Vtk::StreamLine.new
sl.SetInputConnection(pl3d.GetOutputPort)
sl.SetSource(rake.GetOutput)
sl.SetIntegrator(integ)
sl.SetMaximumPropagationTime(0.1)
sl.SetIntegrationStepLength(0.1)
sl.SetIntegrationDirectionToBackward
sl.SetStepLength(0.001)

# The ruled surface stiches together lines with triangle strips.
# Note the SetOnRatio method. It turns on every other strip that
# the filter generates (only when multiple lines are input).
scalarSurface = Vtk::RuledSurfaceFilter.new
scalarSurface.SetInputConnection(sl.GetOutputPort)
scalarSurface.SetOffset(0)
scalarSurface.SetOnRatio(2)
scalarSurface.PassLinesOn
scalarSurface.SetRuledModeToPointWalk
scalarSurface.SetDistanceFactor(30)
mapper = Vtk::PolyDataMapper.new
mapper.SetInputConnection(scalarSurface.GetOutputPort)
mapper.SetScalarRange(pl3d.GetOutput.GetScalarRange)
actor = Vtk::Actor.new
actor.SetMapper(mapper)

# Put an outline around for context.
outline = Vtk::StructuredGridOutlineFilter.new
outline.SetInputConnection(pl3d.GetOutputPort)
outlineMapper = Vtk::PolyDataMapper.new
outlineMapper.SetInputConnection(outline.GetOutputPort)
outlineActor = Vtk::Actor.new
outlineActor.SetMapper(outlineMapper)
outlineActor.GetProperty.SetColor(0, 0, 0)

# Now create the usual graphics stuff.
ren = Vtk::Renderer.new
renWin = Vtk::RenderWindow.new
renWin.AddRenderer(ren)
iren = Vtk::RenderWindowInteractor.new
iren.SetRenderWindow(renWin)

ren.AddActor(rakeActor)
ren.AddActor(actor)
ren.AddActor(outlineActor)
ren.SetBackground(1, 1, 1)

renWin.SetSize(300, 300)

iren.Initialize
renWin.Render
iren.Start
