#!/usr/bin/env ruby

# This is a simple volume rendering example that uses a
# vtkVolumeRayCast mapper

require 'vtk'
require 'vtk/util'
VTK_DATA_ROOT = vtkGetDataRoot

# Create the standard renderer, render window and interactor
ren = Vtk::Renderer.new
renWin = Vtk::RenderWindow.new
renWin.AddRenderer(ren)
iren = Vtk::RenderWindowInteractor.new
iren.SetRenderWindow(renWin)

# Create the reader for the data
reader = Vtk::StructuredPointsReader.new
reader.SetFileName(VTK_DATA_ROOT + "/Data/ironProt.vtk")


# Create transfer mapping scalar value to opacity
opacityTransferFunction = Vtk::PiecewiseFunction.new
opacityTransferFunction.AddPoint(20, 0.0)
opacityTransferFunction.AddPoint(255, 0.2)

# Create transfer mapping scalar value to color
colorTransferFunction = Vtk::ColorTransferFunction.new
colorTransferFunction.AddRGBPoint(0.0, 0.0, 0.0, 0.0)
colorTransferFunction.AddRGBPoint(64.0, 1.0, 0.0, 0.0)
colorTransferFunction.AddRGBPoint(128.0, 0.0, 0.0, 1.0)
colorTransferFunction.AddRGBPoint(192.0, 0.0, 1.0, 0.0)
colorTransferFunction.AddRGBPoint(255.0, 0.0, 0.2, 0.0)

# The property describes how the data will look
volumeProperty = Vtk::VolumeProperty.new
volumeProperty.SetColor(colorTransferFunction)
volumeProperty.SetScalarOpacity(opacityTransferFunction)
volumeProperty.ShadeOn
volumeProperty.SetInterpolationTypeToLinear

# The mapper / ray cast function know how to render the data
compositeFunction = Vtk::VolumeRayCastCompositeFunction.new
volumeMapper = Vtk::VolumeRayCastMapper.new
volumeMapper.SetVolumeRayCastFunction(compositeFunction)
volumeMapper.SetInputConnection(reader.GetOutputPort)

# The volume holds the mapper and the property and
# can be used to position/orient the volume
volume = Vtk::Volume.new
volume.SetMapper(volumeMapper)
volume.SetProperty(volumeProperty)

ren.AddVolume(volume)
ren.SetBackground(1, 1, 1)
renWin.SetSize(600, 600)
renWin.Render


CheckAbort = Proc.new{|obj, event|
  if obj.GetEventPending != 0
    obj.SetAbortRender(1)
  end
}
 
renWin.AddObserver("AbortCheckEvent", CheckAbort)

iren.Initialize
renWin.Render
iren.Start
